/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.0 of the PHP license,       |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_0.txt.                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Ilia Alshanetsky <ilia@php.net>                              |
  +----------------------------------------------------------------------+
*/

/* $Id: statgrab.c,v 1.1 2004/01/28 03:24:44 iliaa Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <statgrab.h>

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "ext/standard/php_string.h" /* needed for php_explode */
#include "php_statgrab.h"

#define PHP_STATGRAB_VERSION "0.1"

#define PHP_STATG_ADD_FVAL(rtz, key, val)	add_assoc_double_ex(rtz, key, sizeof(key), val);
#define PHP_STATG_ADD_LVAL(rtz, key, val)	add_assoc_long_ex(rtz, key, sizeof(key), val);
#define PHP_STATG_ADD_LLVAL(rtz, key, val)	{	\
	char tmp[256];	\
	int tmp_len = snprintf((char *)&tmp, sizeof(tmp) - 1, "%lld", val);	\
	add_assoc_stringl_ex(rtz, key, sizeof(key), tmp, tmp_len, 1);	\
}


/* True global resources - no need for thread safety here 
static int le_statgrab;
*/

/* {{{ statgrab_functions[]
 *
 * Every user visible function must have an entry in statgrab_functions[].
 */
function_entry statgrab_functions[] = {
	PHP_FE(statg_cpu_percent_usage,	NULL)
	PHP_FE(statg_cpu_totals,	NULL)
	PHP_FE(statg_cpu_diff,		NULL)
	PHP_FE(statg_diskio_stats,	NULL)
	PHP_FE(statg_diskio_stats_diff,	NULL)
	PHP_FE(statg_general_stats,	NULL)
	PHP_FE(statg_load_stats,	NULL)
	PHP_FE(statg_memory_stats,	NULL)
	PHP_FE(statg_swap_stats,	NULL)
	PHP_FE(statg_network_stats,	NULL)
	PHP_FE(statg_network_stats_diff,NULL)
	PHP_FE(statg_page_stats,	NULL)
	PHP_FE(statg_page_stats_diff,	NULL)
	PHP_FE(statg_process_stats,	NULL)
	PHP_FE(statg_user_stats,	NULL)
	{NULL, NULL, NULL}	/* Must be the last line in statgrab_functions[] */
};
/* }}} */

/* {{{ statgrab_module_entry
 */
zend_module_entry statgrab_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"statgrab",
	statgrab_functions,
	PHP_MINIT(statgrab),
	NULL,
	NULL,
	NULL,
	PHP_MINFO(statgrab),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1", /* Replace with version number for your extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_STATGRAB
ZEND_GET_MODULE(statgrab)
#endif

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(statgrab)
{
	/* Initialise statgrab */
	statgrab_init();
	                
	/* Drop setuid/setgid privileges. */
        if (statgrab_drop_privileges() != 0) {
		zend_error(E_ERROR, "libstatgrab: Failed to drop privileges");
	}                                                

	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(statgrab)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "statgrab support", "enabled");
	php_info_print_table_header(2, "Extension version", PHP_STATGRAB_VERSION " - $id$");
	php_info_print_table_end();
}
/* }}} */

/* {{{ proto array statg_cpu_percent_usage()
   Return an array indicating the cpu utilization */
PHP_FUNCTION(statg_cpu_percent_usage)
{
	cpu_percent_t *cpu;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	cpu = cpu_percent_usage();

	array_init(return_value);

	PHP_STATG_ADD_FVAL(return_value, "user", cpu->user)
	PHP_STATG_ADD_FVAL(return_value, "kernel", cpu->kernel)
	PHP_STATG_ADD_FVAL(return_value, "idle", cpu->idle)
	PHP_STATG_ADD_FVAL(return_value, "iowait", cpu->iowait)
	PHP_STATG_ADD_FVAL(return_value, "swap", cpu->swap)
	PHP_STATG_ADD_FVAL(return_value, "nice", cpu->nice)
	PHP_STATG_ADD_LVAL(return_value, "previous_run", cpu->time_taken)
}
/* }}} */

static void _php_cpu_stats(INTERNAL_FUNCTION_PARAMETERS, int mode)
{
	cpu_states_t *cpu;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	if (mode) {
		cpu = get_cpu_totals();
	} else {
		cpu = get_cpu_diff();
	}
	if (cpu == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	PHP_STATG_ADD_LLVAL(return_value, "user", cpu->user)
	PHP_STATG_ADD_LLVAL(return_value, "kernel", cpu->kernel)
	PHP_STATG_ADD_LLVAL(return_value, "idle", cpu->idle)
	PHP_STATG_ADD_LLVAL(return_value, "iowait", cpu->iowait)
	PHP_STATG_ADD_LLVAL(return_value, "swap", cpu->swap)
	PHP_STATG_ADD_LLVAL(return_value, "nice", cpu->nice)
	PHP_STATG_ADD_LVAL(return_value, "previous_run", cpu->systime)
}

/* {{{ proto array statg_cpu_totals()
     Returns the total amount of "ticks" the system has spent in each of the different states. */
PHP_FUNCTION(statg_cpu_totals)
{
	_php_cpu_stats(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
}
/* }}} */

/* {{{ proto array statg_get_cpu_diff()
     Returns the total amount of "ticks" the system has spent in each of the different states since last run. */
PHP_FUNCTION(statg_cpu_diff)
{
	_php_cpu_stats(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
}
/* }}} */

static void _php_statg_diskio(INTERNAL_FUNCTION_PARAMETERS, int mode)
{
	diskio_stat_t *dst;
	int ent, i;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	if (mode) {
		dst = get_diskio_stats(&ent);
	} else {
		dst = get_diskio_stats_diff(&ent);
	}
	if (dst == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	for (i = 0; i < ent; i++) {
		zval *tmpz;

		MAKE_STD_ZVAL(tmpz);
		array_init(tmpz);

		PHP_STATG_ADD_LLVAL(tmpz, "read", dst->read_bytes)
		PHP_STATG_ADD_LLVAL(tmpz, "written", dst->write_bytes)
		PHP_STATG_ADD_LVAL(tmpz, "time_frame", dst->systime)

		add_assoc_zval(return_value, dst->disk_name, tmpz);

		dst++;
	}
}

/* {{{ proto array statg_diskio_stats()
     Returns the disk IO stored in the kernel which holds the amount of data transferred since bootup. */
PHP_FUNCTION(statg_diskio_stats)
{
	_php_statg_diskio(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
}
/* }}} */

/* {{{ proto array statg_diskio_stats_diff()
     Returns the disk IO stored in the kernel which holds the amount of data transferred since last call. */
PHP_FUNCTION(statg_diskio_stats_diff)
{
	_php_statg_diskio(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
}
/* }}} */

/* {{{ proto array statg_general_stats()
     Returns general information about the operating system. */
PHP_FUNCTION(statg_general_stats)
{
	general_stat_t *stats;
	
	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	stats = get_general_stats();
	if (stats == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	add_assoc_string(return_value, "os_name", stats->os_name, 1);
	add_assoc_string(return_value, "os_release", stats->os_release, 1);
	add_assoc_string(return_value, "os_version", stats->os_version, 1);
	add_assoc_string(return_value, "platform", stats->platform, 1);
	add_assoc_string(return_value, "hostname", stats->hostname, 1);
	PHP_STATG_ADD_LVAL(return_value, "uptime", stats->uptime)
}
/* }}} */

/* {{{ proto array statg_load_stats()
     Returns the average load for the last 1, 5 and 15 minutes. */
PHP_FUNCTION(statg_load_stats)
{
	load_stat_t *lst;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	lst = get_load_stats();
	if (lst == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	PHP_STATG_ADD_FVAL(return_value, "min1", lst->min1)
	PHP_STATG_ADD_FVAL(return_value, "min5", lst->min5)
	PHP_STATG_ADD_FVAL(return_value, "min15", lst->min15)
}
/* }}} */

/* {{{ proto array statg_memory_stats()
     Returns information about memory usage. */
PHP_FUNCTION(statg_memory_stats)
{
	mem_stat_t *ms;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	ms = get_memory_stats();
	if (ms == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	PHP_STATG_ADD_LLVAL(return_value, "total", ms->total)
	PHP_STATG_ADD_LLVAL(return_value, "free", ms->free)
	PHP_STATG_ADD_LLVAL(return_value, "used", ms->used)
	PHP_STATG_ADD_LLVAL(return_value, "cache", ms->cache)
}
/* }}} */

/* {{{ proto array statg_swap_stats()
     Returns information about swap usage. */
PHP_FUNCTION(statg_swap_stats)
{
	swap_stat_t *ss;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	ss = get_swap_stats();
	if (ss == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	PHP_STATG_ADD_LLVAL(return_value, "total", ss->total)
	PHP_STATG_ADD_LLVAL(return_value, "free", ss->free)
	PHP_STATG_ADD_LLVAL(return_value, "used", ss->used)
}
/* }}} */

static void _php_statg_network(INTERNAL_FUNCTION_PARAMETERS, int mode)
{
	network_stat_t *ns;
	int ent, i;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	if (mode) {
		ns = get_network_stats(&ent);
	} else {
		ns = get_network_stats_diff(&ent);
	}
	if (ns == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);
	
	for (i = 0; i < ent; i++) {
		zval *tmpz;

		MAKE_STD_ZVAL(tmpz);
		array_init(tmpz);

		PHP_STATG_ADD_LLVAL(tmpz, "sent", ns->tx)
		PHP_STATG_ADD_LLVAL(tmpz, "recieved", ns->rx)
		PHP_STATG_ADD_LVAL(tmpz, "time_frame", ns->systime)

		add_assoc_zval(return_value, ns->interface_name, tmpz);

		ns++;
	}
}

/* {{{ proto array statg_network_stats()
     Returns an array containing transmition statistics for all interfaces. */
PHP_FUNCTION(statg_network_stats)
{
	_php_statg_network(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
}
/* }}} */

/* {{{ proto array statg_network_stats_diff()
     Returns an array containing transmition statistics for all interfaces since last call. */
PHP_FUNCTION(statg_network_stats_diff)
{
	_php_statg_network(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
}
/* }}} */

static void _php_statg_pages(INTERNAL_FUNCTION_PARAMETERS, int mode)
{
	page_stat_t *ps;
	int ent;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	if (mode) {
		ps = get_page_stats((int)ent);
	} else {
		ps = get_page_stats_diff((int)ent);
	}
	if (ps == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	PHP_STATG_ADD_LLVAL(return_value, "pages_in", ps->pages_pagein)
	PHP_STATG_ADD_LLVAL(return_value, "pages_out", ps->pages_pageout)
	PHP_STATG_ADD_LVAL(return_value, "time_frame", ps->systime)
}

/* {{{ proto array statg_page_stats()
     Return the number of pages the system has paged in and out since bootup. */
PHP_FUNCTION(statg_page_stats)
{
	_php_statg_pages(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
}
/* }}} */

/* {{{ proto array statg_page_stats_diff()
     Return the number of pages the system has paged in and out since last call to the function. */
PHP_FUNCTION(statg_page_stats_diff)
{
	_php_statg_pages(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
}
/* }}} */

/* {{{ proto array statg_process_stats()
     Returns information about the currently running processes. */
PHP_FUNCTION(statg_process_stats)
{
	process_stat_t *ps;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	ps = get_process_stats();
	if (ps == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	PHP_STATG_ADD_LVAL(return_value, "total", ps->total)
	PHP_STATG_ADD_LVAL(return_value, "running", ps->running)
	PHP_STATG_ADD_LVAL(return_value, "sleeping", ps->sleeping)
	PHP_STATG_ADD_LVAL(return_value, "stopped", ps->stopped)
	PHP_STATG_ADD_LVAL(return_value, "zombie", ps->zombie)
}
/* }}} */

/* {{{ proto array statg_user_stats()
     Returns an array containing all of the currently logged in users. */
PHP_FUNCTION(statg_user_stats)
{
	user_stat_t *us;
	zval delim, str;

	if (ZEND_NUM_ARGS()) {
		WRONG_PARAM_COUNT;
	}

	us = get_user_stats();
	if (us == NULL) {
		RETURN_FALSE;
	}

	array_init(return_value);

	ZVAL_STRINGL(&delim, " ", sizeof(" ") - 1, 0);
	ZVAL_STRINGL(&str, us->name_list, strlen(us->name_list), 0);

	php_explode(&delim, &str, return_value, us->num_entries);
}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
